#
#  Name:    mx.tcl
#
#  Purpose: Tcl interface to MX.
#
#  Author:  William Lavender.
#
#-----------------------------------------------------------------------
#
#  Copyright 1999-2003 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#


package provide Mx 0.65

namespace eval Mx {
	variable user_interrupt 0
}

#
#  Namespace procedures
#

proc Mx::init {} {
	mxprivate_init
}

proc Mx::object_name { record_name } {
	set object_name "::mxrecord_"

	append object_name $record_name

	return $object_name
}

proc Mx::get_debug_level {} {
	mxprivate_get_debug_level
}

proc Mx::set_debug_level { debug_level } {
	mxprivate_set_debug_level $debug_level
}

proc Mx::get_version {} {
	mxprivate_get_version
}

proc Mx::process_exists { process_id } {
	set status [ mxprivate_process_exists $process_id ]

	return $status
}

#
# Error message box
#

proc Mx::error_message { title_bar explanatory_message \
			result_from_catch error_info } {

	if { $title_bar == "" } {
		set title_bar "Bug: No error dialog title bar provided"
	}

	if { $explanatory_message == "" } {
		set explanatory_message \
"Program Bug: No explanatory message was provided for this error dialog."
	}

	# See if Tk is running.

	set status [ catch { winfo id . } ]

	if { $status != 0 } {

		# Tk is not running, so we must send this message
		# to the user another way.

		Mx::info "Error: $explanatory_message"

		return
	}

	set wraplength 500

	set random [ expr ( round ( 1000000000.0 * rand() ) ) ]

	set errorwin .mx_error_$random

	toplevel $errorwin

	wm title $errorwin $title_bar

	label $errorwin.label -justify left -wraplength $wraplength \
					-text "$explanatory_message"

	pack $errorwin.label -padx 5 -pady 5

	if { $result_from_catch != "" } {
		frame $errorwin.sep1 -height 2 -borderwidth 1 -relief sunken
		pack $errorwin.sep1 -fill x -pady 5

		label $errorwin.reason -justify left -wraplength $wraplength \
					-text "Reason: $result_from_catch"

		pack $errorwin.reason -padx 5 -pady 5
	}

	if { $error_info != "" } {
		frame $errorwin.sep2 -height 2 -borderwidth 1 -relief sunken
		pack $errorwin.sep2 -fill x -pady 5

		label $errorwin.error_info -justify left \
					-wraplength $wraplength \
					-text "Stack Trace:\n$error_info"

		pack $errorwin.error_info -padx 5 -pady 5 -anchor w
	}

	frame $errorwin.sep3 -height 2 -borderwidth 1 -relief sunken
	pack $errorwin.sep3 -fill x -pady 5

	button $errorwin.ok -text "OK" -command [ itcl::code destroy $errorwin ]
	pack $errorwin.ok -padx 5 -pady 5

	frame $errorwin.sep4 -height 2 -borderwidth 1 -relief flat
	pack $errorwin.sep4 -fill x -pady 2

	tkwait window $errorwin
}

#
# User interrupt functions
#

proc Mx::getch {} {
	mxprivate_getch
}

proc Mx::kbhit {} {
	mxprivate_kbhit
}

proc Mx::user_requested_interrupt {} {
	mxprivate_user_requested_interrupt
}

proc Mx::set_user_interrupt_function { function } {
	mxprivate_set_user_interrupt_function $function
}

#
# Output functions
#

proc Mx::info { message } {
	mxprivate_info $message
}

proc Mx::set_info_output_function { function } {
	mxprivate_set_info_output_function $function
}

proc Mx::warning { message } {
	mxprivate_warning $message
}

proc Mx::set_warning_output_function { function } {
	mxprivate_set_warning_output_function $function
}

proc Mx::set_error_output_function { function } {
	mxprivate_set_error_output_function $function
}

proc Mx::debug { message } {
	mxprivate_debug $message
}

proc Mx::set_debug_output_function { function } {
	mxprivate_set_debug_output_function $function
}

proc Mx::set_all_output_functions { function } {
	Mx::set_info_output_function $function
	Mx::set_warning_output_function $function
	Mx::set_error_output_function $function
	Mx::set_debug_output_function $function
}

#
# Info dialog functions
#

proc Mx::info_dialog { text_prompt gui_prompt button_label } {
	mxprivate_info_dialog $text_prompt $gui_prompt $button_label
}

proc Mx::set_info_dialog_function { function } {
	mxprivate_set_info_dialog_function $function
}

#
#  Setup_database is a utility function to simplify the startup
#  of MxTcl programs.
#

proc Mx::setup_database { filename } {
	#
	#  Create a new record list.
	#

	set record_list [ Mx::RecordList ::mxrecordlist_#auto ]

	#
	#  Enable plotting.
	#

	$record_list set_plot_enable 1

	#
	#  Read the database in and initialize the corresponding hardware.
	#

	$record_list read_database $filename

	$record_list finish_database_initialization

	$record_list initialize_hardware 0

	$record_list configure -unsaved_scans 0

	return $record_list
}

#
#  Mx::RecordList class
#

itcl::class Mx::RecordList {
	public variable list_handle -1
	public variable list_head_record ""

	public variable unsaved_scans 0

	constructor {} {
		set list_handle [ mxprivate_create_record_list ]

		set record_name [ eval mxprivate_get_record_name $list_handle ]

		set object_name [ Mx::object_name $record_name ]

		set list_head_record [ Mx::Record $object_name ]

		$list_head_record configure -handle $list_handle
		$list_head_record configure -name $record_name
	}
	destructor {
	}

	method read_database { filename }
	method finish_database_initialization {}
	method initialize_hardware { inithw_flags }
	method create_record_from_description { description }

	method get_record { record_name }
	method get_record_quiet { record_name }

	method get_multiple_records \
		{ mx_superclass mx_class mx_type reverse_order }

	method load_scans { filename initialize_flag }
	method save_scans { filename }

	method plotting_is_enabled {}
	method set_plot_enable { plot_enable_value }

	method set_program_name { program_name }

	method log_open {}
	method log_close {}

	private method get_record_main { record_name quiet_flag }
}

itcl::body Mx::RecordList::read_database { filename } {
	mxprivate_read_database $list_handle $filename
}

itcl::body Mx::RecordList::finish_database_initialization {} {
	mxprivate_finish_database_initialization $list_handle
}

itcl::body Mx::RecordList::initialize_hardware { inithw_flags } {
	mxprivate_initialize_hardware $list_handle $inithw_flags
}

itcl::body Mx::RecordList::create_record_from_description { description } {
	mxprivate_create_record_from_description $list_handle $description

	set record_name [lindex [split $description] 0]

	set record [ Mx::RecordList::get_record $record_name ]

	return $record
}

itcl::body Mx::RecordList::get_record { record_name } {
	return [ $this get_record_main $record_name 0 ]
}

itcl::body Mx::RecordList::get_record_quiet { record_name } {
	return [ $this get_record_main $record_name 1 ]
}

itcl::body Mx::RecordList::get_record_main { record_name quiet_flag } {
	set record_handle [ eval mxprivate_get_record \
				$list_handle $record_name $quiet_flag ]

	set object_class [ eval mxprivate_get_object_class_name \
					$record_handle ]

	set object_name [ Mx::object_name $record_name ]

	if [catch { $object_class $object_name } result] {
		set record $object_name
	} else {
		set record $result

		$record configure -handle $record_handle
		$record configure -name $record_name
	}

	return $record
}

itcl::body Mx::RecordList::get_multiple_records \
		{ mx_superclass mx_class mx_type reverse_order } {

	if { $reverse_order == 0 } {
		set current_record [ $list_head_record get_next_record ]
	} else {
		set current_record [ $list_head_record get_previous_record ]
	}

	set record_tcl_list [ list ]

	while { $current_record != $list_head_record } {
		set current_name [ eval $current_record cget -name ]

		set current_superclass \
			[ eval $current_record get_field mx_superclass ]

		if [ regexp -- "$mx_superclass" $current_superclass ] {

			set current_class \
				[ eval $current_record get_field mx_class ]

			if [ regexp -- "$mx_class" $current_class ] {

				set current_type \
				    [ eval $current_record get_field mx_type ]

				if [ regexp -- "$mx_type" $current_type ] {
					lappend record_tcl_list $current_name
				}
			}
		}
		if { $reverse_order == 0 } {
			set current_record [ $current_record get_next_record ]
		} else {
			set current_record \
				[ $current_record get_previous_record ]
		}
	}
	return $record_tcl_list
}

itcl::body Mx::RecordList::load_scans { filename initialize_flag } {

	if [ catch { open $filename r } file_id ] {

		Mx::error_message "Error Loading Scans" \
			"Cannot load scans from database file '$filename'" \
			$file_id ""

		return -code error
	}

	while { 1 } {
		# Read in a record description.

		if [ catch { gets $file_id } description ] {
			log_puts "Error reading description."

			# Exit the while loop.

			break
		}
		set status [ eof $file_id ]

		if { $status != 0 } {
			# Exit the while loop.

			break
		}

		# What is the name of the record we are trying to create?

		set record_name [ lindex [ split $description ] 0 ]

		# Does a record by this name already exist?

		set status [ catch \
			{ $this get_record_quiet $record_name } old_record ]

		if { $status == 0 } {
			# Is the old record a scan record?

			set superclass [ $old_record get_field mx_superclass ]

			if { $superclass == "scan" } {
				set t \
				    "Scan '$record_name' from file '$filename' "
				append t \
				    "will replace old scan '$record_name'."

#
#				log_puts $t
#

				$old_record delete_record

			} else {
				log_puts \
			"Skipping scan '$record_name' from file '$filename'.  "
				log_puts \
	"Only preexisting records that are scan records may be replaced."

				continue
			}
		}

		# Now create the record.

		if [catch { $this create_record_from_description \
					$description } new_record] {

			log_puts "Error creating record '$record_name'."

			close $file_id
			return -code error
		}
		$this configure -unsaved_scans 1

		if { $initialize_flag != 0 } {

			if [catch { $new_record \
					finish_record_initialization } ] {

				log_puts "Error initializing record '$record_name'."

				close $file_id
				return -code error
			}
		}
	}
	close $file_id
}

itcl::body Mx::RecordList::save_scans { filename } {

	if [ catch { open $filename w } file_id ] {

		Mx::error_message "Error Saving Scans" \
			"Cannot save scans to database file '$filename'" \
			$file_id ""

		return -code error
	}

	set current_record [ $list_head_record get_next_record ]

	while { $current_record != $list_head_record } {

		set superclass [ eval $current_record get_field mx_superclass ]

		if { $superclass == "scan" } {

			set description [ $current_record \
					create_description_from_record ]

			puts $file_id $description
		}
		set current_record [ $current_record get_next_record ]
	}
	close $file_id

	$this configure -unsaved_scans 0
}

itcl::body Mx::RecordList::plotting_is_enabled {} {
	set plot_enable_value [ mxprivate_plotting_is_enabled $list_handle ]

	return $plot_enable_value
}

itcl::body Mx::RecordList::set_plot_enable { plot_enable_value } {
	mxprivate_set_plot_enable $list_handle $plot_enable_value
}

itcl::body Mx::RecordList::set_program_name { program_name } {
	mxprivate_set_program_name $list_handle $program_name
}

itcl::body Mx::RecordList::log_open {} {
	mxprivate_log_open $list_handle
}

itcl::body Mx::RecordList::log_close {} {
	mxprivate_log_close $list_handle
}


#
#  Mx::Record class
#

itcl::class Mx::Record {
	public variable handle -1
	public variable name ""

	constructor {} {
	}
	destructor {
	}

	method create_description_from_record {}
	method finish_record_initialization {}
	method delete_record {}
	method get_next_record {}
	method get_previous_record {}
	method get_field { field_name }
	method set_field { field_name value }
}

itcl::body Mx::Record::create_description_from_record {} {
	mxprivate_create_description_from_record $handle
}

itcl::body Mx::Record::finish_record_initialization {} {
	mxprivate_finish_record_initialization $handle
}

itcl::body Mx::Record::delete_record {} {
	mxprivate_delete_record $handle

	itcl::delete object $this
}

itcl::body Mx::Record::get_next_record {} {
	set next_record_handle [ eval mxprivate_get_next_record $handle ]

	set next_record_name [ eval mxprivate_get_record_name \
					$next_record_handle ]

	set next_object_class [ eval mxprivate_get_object_class_name \
					$next_record_handle ]

	set next_object_name [ Mx::object_name $next_record_name ]

	if [catch { $next_object_class $next_object_name } result] {
		set next_record $next_object_name
	} else {
		set next_record $result

		$next_record configure -handle $next_record_handle
		$next_record configure -name $next_record_name
	}

	return $next_record
}

itcl::body Mx::Record::get_previous_record {} {
	set previous_record_handle \
				[ eval mxprivate_get_previous_record $handle ]

	set previous_record_name [ eval mxprivate_get_record_name \
					$previous_record_handle ]

	set previous_object_class [ eval mxprivate_get_object_class_name \
					$previous_record_handle ]

	set previous_object_name [ Mx::object_name $previous_record_name ]

	if [catch { $previous_object_class $previous_object_name } result] {
		set previous_record $previous_object_name
	} else {
		set previous_record $result

		$previous_record configure -handle $previous_record_handle
		$previous_record configure -name $previous_record_name
	}

	return $previous_record
}

itcl::body Mx::Record::get_field { field_name } {
	set type [ eval mxprivate_get_field $handle $field_name ]
}

itcl::body Mx::Record::set_field { field_name string_value } {
	mxprivate_set_field $handle $field_name $string_value
}

#
#  Mx::RS232 class
#

itcl::class Mx::RS232 {
	inherit Mx::Record

	method getchar {}
	method putchar { c }
	method getline {}
	method putline { line }
	method input_is_available {}
	method discard_unread_input {}
	method discard_unwritten_output {}
}

itcl::body Mx::RS232::getchar {} {
	mxprivate_rs232_getchar $handle
}

itcl::body Mx::RS232::putchar { c } {
	mxprivate_rs232_putchar $handle $c
}

itcl::body Mx::RS232::getline {} {
	mxprivate_rs232_getline $handle
}

itcl::body Mx::RS232::putline { line } {
	mxprivate_rs232_putline $handle $line
}

itcl::body Mx::RS232::input_is_available {} {
	mxprivate_rs232_input_is_available $handle
}

itcl::body Mx::RS232::discard_unread_input {} {
	mxprivate_rs232_discard_unread_input $handle
}

itcl::body Mx::RS232::discard_unwritten_output {} {
	mxprivate_rs232_discard_unwritten_output $handle
}

#
#  Mx::DigitalInput class
#

itcl::class Mx::DigitalInput {
	inherit Mx::Record

	method read {}
}

itcl::body Mx::DigitalInput::read {} {
	mxprivate_digital_input_read $handle
}

#
#  Mx::DigitalOutput class
#

itcl::class Mx::DigitalOutput {
	inherit Mx::Record

	method read {}
	method write { value }
}

itcl::body Mx::DigitalOutput::read {} {
	mxprivate_digital_output_read $handle
}

itcl::body Mx::DigitalOutput::write { value } {
	mxprivate_digital_output_write $handle $value
}

#
#  Mx::Motor class
#

itcl::class Mx::Motor {
	inherit Mx::Record

	public variable tweak_distance 0

	method is_busy {}
	method move_absolute { new_destination }
	method get_position {}
	method set_position { new_set_position }
	method soft_abort {}
	method immediate_abort {}
	method positive_limit_hit {}
	method negative_limit_hit {}
	method find_home_position { direction }
	method constant_velocity_move { direction }
	method get_speed {}
	method set_speed { speed }
	method get_base_speed {}
	method set_base_speed { speed }
	method get_maximum_speed {}
	method set_maximum_speed { speed }
	method get_acceleration_type {}
	method get_raw_acceleration_parameters {}
	method set_raw_acceleration_parameters { parameter_list }
	method home_search_succeeded {}
}

itcl::body Mx::Motor::is_busy {} {
	mxprivate_motor_is_busy $handle
}

itcl::body Mx::Motor::move_absolute { new_destination } {
	mxprivate_motor_move_absolute $handle $new_destination
}

itcl::body Mx::Motor::get_position {} {
	mxprivate_motor_get_position $handle
}

itcl::body Mx::Motor::set_position { new_set_position } {
	mxprivate_motor_set_position $handle $new_set_position
}

itcl::body Mx::Motor::soft_abort {} {
	mxprivate_motor_soft_abort $handle
}

itcl::body Mx::Motor::immediate_abort {} {
	mxprivate_motor_immediate_abort $handle
}

itcl::body Mx::Motor::positive_limit_hit {} {
	mxprivate_motor_positive_limit_hit $handle
}

itcl::body Mx::Motor::negative_limit_hit {} {
	mxprivate_motor_negative_limit_hit $handle
}

itcl::body Mx::Motor::find_home_position { direction } {
	mxprivate_motor_find_home_position $handle $direction
}

itcl::body Mx::Motor::constant_velocity_move { direction } {
	mxprivate_motor_constant_velocity_move $handle $direction
}

itcl::body Mx::Motor::get_speed {} {
	mxprivate_motor_get_speed $handle
}

itcl::body Mx::Motor::set_speed { speed } {
	mxprivate_motor_set_speed $handle $speed
}

itcl::body Mx::Motor::get_base_speed {} {
	mxprivate_motor_get_base_speed $handle
}

itcl::body Mx::Motor::set_base_speed { speed } {
	mxprivate_motor_set_base_speed $handle $speed
}

itcl::body Mx::Motor::get_maximum_speed {} {
	mxprivate_motor_get_maximum_speed $handle
}

itcl::body Mx::Motor::set_maximum_speed { speed } {
	mxprivate_motor_set_maximum_speed $handle $speed
}

itcl::body Mx::Motor::get_acceleration_type {} {
	mxprivate_motor_get_acceleration_type $handle
}

itcl::body Mx::Motor::get_raw_acceleration_parameters {} {
	mxprivate_motor_get_raw_acceleration_parameters $handle
}

itcl::body Mx::Motor::set_raw_acceleration_parameters { parameter_list } {
	mxprivate_motor_set_raw_acceleration_parameters $handle $parameter_list
}

itcl::body Mx::Motor::home_search_succeeded {} {
	mxprivate_motor_home_search_succeeded $handle
}

#
#  Mx::Relay class
#

itcl::class Mx::Relay {
	inherit Mx::Record

	method relay_command { command }
	method get_relay_status {}
}

itcl::body Mx::Relay::relay_command { command } {
	mxprivate_relay_command $handle $command
}

itcl::body Mx::Relay::get_relay_status {} {
	return [ mxprivate_get_relay_status $handle ]
}

#
#  Mx::Scaler class
#

itcl::class Mx::Scaler {
	inherit Mx::Record

	method clear {}
	method overflow_set {}
	method read {}
	method read_raw {}
	method write { value }
	method get_dark_current {}
	method set_dark_current { dark_current }
}

itcl::body Mx::Scaler::clear {} {
	mxprivate_scaler_clear $handle
}

itcl::body Mx::Scaler::overflow_set {} {
	mxprivate_scaler_overflow_set $handle
}

itcl::body Mx::Scaler::read {} {
	mxprivate_scaler_read $handle
}

itcl::body Mx::Scaler::read_raw {} {
	mxprivate_scaler_read_raw $handle
}

itcl::body Mx::Scaler::write { value } {
	mxprivate_scaler_write $handle $value
}

itcl::body Mx::Scaler::get_dark_current {} {
	mxprivate_scaler_get_dark_current $handle
}

itcl::body Mx::Scaler::set_dark_current { dark_current } {
	mxprivate_scaler_set_dark_current $handle $dark_current
}

#
#  Mx::Timer class
#

itcl::class Mx::Timer {
	inherit Mx::Record

	method is_busy {}
	method start { seconds }
	method stop {}
}

itcl::body Mx::Timer::is_busy {} {
	mxprivate_timer_is_busy $handle
}

itcl::body Mx::Timer::start { seconds } {
	mxprivate_timer_start $handle $seconds
}

itcl::body Mx::Timer::stop {} {
	mxprivate_timer_stop $handle
}

#
#  Mx::Scan class
#

itcl::class Mx::Scan {
	inherit Mx::Record

	method perform_scan {}
}

itcl::body Mx::Scan::perform_scan {} {
	mxprivate_scan_perform_scan $handle
}

#
#  Mx::Variable class
#

itcl::class Mx::Variable {
	inherit Mx::Record

	method send_variable { value }
	method receive_variable {}
}

itcl::body Mx::Variable::send_variable { value } {
	mxprivate_send_variable $handle $value
}

itcl::body Mx::Variable::receive_variable {} {

	return [ mxprivate_receive_variable $handle ]
}

